unit F_Main;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Objects,
  FMX.Controls.Presentation, FMX.StdCtrls, math, FMX.Layouts, FMX.Ani,
  FMX.Gestures, System.Actions, FMX.ActnList;

type

  tpDirezione = (tpBasso, tpAlto, tpDestra, tpSinistra);

  TPlayTile = Class(TRectangle)
  Private
    fnValore: Integer;
  Public
    Property nValore: Integer read fnValore write fnValore;
  End;

  TFMain = class(TForm)
    Layout1: TLayout;
    btsinistra: TRectangle;
    btAlto: TRectangle;
    btDestra: TRectangle;
    btBasso: TRectangle;
    FloatAnimation2: TFloatAnimation;
    FloatAnimation3: TFloatAnimation;
    FloatAnimation4: TFloatAnimation;
    FloatAnimation5: TFloatAnimation;
    RectAnimation1: TRectAnimation;
    RectAnimation2: TRectAnimation;
    RectAnimation3: TRectAnimation;
    FloatAnimation6: TFloatAnimation;
    RectAnimation4: TRectAnimation;
    RectAnimation5: TRectAnimation;
    GradientAnimation1: TGradientAnimation;
    lPunteggio: TLabel;
    Label2: TLabel;
    gbGrid: TGroupBox;
    rb4: TRadioButton;
    rb6: TRadioButton;
    rb8: TRadioButton;
    lMosse: TLabel;
    Label3: TLabel;
    Rectangle2: TRectangle;
    Rectangle3: TRectangle;
    lMossefatte: TLabel;
    Label4: TLabel;
    procedure Rectangle1Paint(Sender: TObject; Canvas: TCanvas;
      const [Ref] ARect: TRectF);
    procedure btBassoClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btAltoClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
    procedure btDestraClick(Sender: TObject);
    procedure btsinistraClick(Sender: TObject);
    procedure gbGridClick(Sender: TObject);
    procedure Rectangle1Click(Sender: TObject);
    procedure Rectangle3Click(Sender: TObject);
  private
    aGrid: Array[1..8,1..8] of integer;
    fnPunteggio: Integer;
    fnMosse: Integer;
    fnMosseFatte: Integer;
    fnCellRow: Integer;
    procedure CleanObj;
    procedure DrawMap(nCell: Integer);
    procedure FillRect;
    procedure NewNumber;
    procedure SetnPunteggio(const Value: Integer);
    function GetnPunteggio: Integer;
    procedure CleanArray;
    procedure MoveSXDX(nLimite, nFrom, nPasso: Integer; oDirezione: tpDirezione);
    function GetnMosse: Integer;
    procedure SetnMosse(const Value: Integer);
    Function SetColor(nNum: Integer): TAlphaColor;
    Procedure HighScore(lNew: Boolean);
    Function NoMore:Boolean;
    function GetnMosseFatte: Integer;
    procedure SetnMosseFatte(const Value: Integer);
    { Private declarations }
  public
    Property nPunteggio: Integer read GetnPunteggio write SetnPunteggio;
    Property nMosse: Integer read GetnMosse write SetnMosse;
    Property nMosseFatte: Integer read GetnMosseFatte write SetnMosseFatte;
    Property nCellRow: Integer read fnCellRow write fnCellRow;
    { Public declaations }
  end;

var
  FMain: TFMain;
CONST
  CS_MINROW = 4;
  CS_MEDROW = 6;
  CS_MAXROW = 8;
  CS_GRID_X = 400;
  CS_GRID_Y = 400;
implementation

{$R *.fmx}

uses uHighScores;
{$R *.NmXhdpiPh.fmx ANDROID}
{$R *.XLgXhdpiTb.fmx ANDROID}
{$R *.Macintosh.fmx MACOS}
{$R *.Windows.fmx MSWINDOWS}

Procedure TFMain.CleanObj;
Var nNumberObj: Integer;
  lFind: Boolean;
  oComp: TComponent;
begin
  nNumberObj := 0;
  lFind := True;
  repeat
    oComp := FindComponent('M'+nNumberObj.ToString);
    if assigned(oComp) then
      FreeAndNil(oComp);
    inc(nNumberObj);
  until not Assigned(FindComponent('M'+nNumberObj.ToString));
end;

Procedure TFMain.CleanArray;
var
  y: Integer;
  x: Integer;
begin
  nPunteggio := 0;
  nMosse := 0;
  nMosseFatte := 0;
  for x := 1 to CS_MAXROW do
    for y := 1 to CS_MAXROW do
      aGrid[x][y] := 0;
end;
procedure TFMain.DrawMap(nCell: Integer);
Var z,x,y: Integer;
    oRect: TRectangle;
    nNumberObj: Integer;
begin
// reset oggetti e ridisegno la nuova griglia
  CleanObj;
  nNumberObj := 0;
  for y := 1 TO nCell do
  begin
    FOR x := 1 TO nCell do
    begin
      oRect := TPlayTile.Create(Self);
      oRect.Parent := Layout1;
      orect.Opacity := 0.1;
      oRect.Canvas.BeginScene;
      oRect.Size.Height := (CS_GRID_Y / nCellRow);
      oRect.Size.Width := (CS_GRID_X / nCellRow);
      oRect.XRadius := 5;
      oRect.YRadius := 5;
      Canvas.Font.Family := 'Modern';
      case nCellRow of
        4:  Canvas.Font.Size := 30;
        6:  Canvas.Font.Size := 25;
        8:  Canvas.Font.Size := 15;
      end;
      orect.canvas.Stroke.Kind := TBrushKind.bkSolid;
      orect.canvas.StrokeThickness := 0;
      oRect.Canvas.Fill.Color := TAlphaColors.Black;
      oRect.Position.X := (x-1)*(CS_GRID_X / nCellRow);
      oRect.Position.Y := (y-1)*(CS_GRID_Y / nCellRow);
      oRect.Name := 'M' + nNumberObj.ToString;
      oRect.Tag := aGrid[y][x];
      Inc(nNumberObj);
      oRect.OnPaint :=  Rectangle1Paint;
      oRect.Fill.Color := TAlphaColors.Azure;
      oRect.Fill.Kind := TBrushKind.bkSolid;
      oRect.Stroke.Thickness := 1;
      oRect.Stroke.Color := TAlphaColors.Darkgray;
      oRect.Canvas.EndScene;
   end;
  end;

end;

procedure TFMain.NewNumber;
Var x,y: Integer;
begin
  while true do
  begin
     x := Random(nCellRow) + 1;
     y := Random(nCellRow) + 1;
     IF aGrid[y][x] = 0 Then Break;
  end;
  nMosse := nMosse + 1;
  nMosseFatte := nMosseFatte + 1;
  if random(9) > 8 then
    aGrid[y][x] := 4
  else
    aGrid[y][x] := 2;
end;

function TFMain.NoMore: Boolean;
var
  j: Integer;
  i: Integer;
begin
  Result := True;
  for j := 1 TO 4 do
    for i := 1 TO 3 do
       if aGrid[j][i] = aGrid[j][i + 1] then Result := False;

  for i := 1 TO 4 do
    for j := 1 TO 3 do
      if aGrid[j][i] = aGrid[j + 1][i] then Result := False;
end;

procedure TFMain.FormKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char;
  Shift: TShiftState);
begin
  if key = vkUp then
    btAltoClick(Nil)
  else if Key = vkDown then
    btBassoClick(Nil)
  else if Key = vkLeft then
    btsinistraClick(Nil)
  else if Key = vkRight then
    btDestraClick(Nil);
end;

procedure TFMain.FormShow(Sender: TObject);
begin
  Randomize;
  fnCellRow := CS_MINROW;
  rb4.IsChecked := True;
end;

procedure TFMain.gbGridClick(Sender: TObject);
begin
  if rb4.IsChecked then
    nCellRow := CS_MINROW
  else if rb6.IsChecked then
    nCellRow := CS_MEDROW
  else if rb8.IsChecked then
    nCellRow := CS_MAXROW;
  CleanArray;
  nMosseFatte := 0;
  NewNumber;
  NewNumber;
  DrawMap(nCellRow);
  FillRect;
end;

function TFMain.GetnMosse: Integer;
begin
  Result := fnMosse;
end;

function TFMain.GetnMosseFatte: Integer;
begin
  Result := fnMosseFatte;
end;

function TFMain.GetnPunteggio: Integer;
begin
  Result := fnPunteggio;
end;

procedure TFMain.HighScore(lNew: Boolean);
Var oHigh: TFrameHighScores;
begin
  oHigh := TFrameHighScores.Create(Self);
  oHigh.InitFrame;
  if lNew then
    oHigh.AddScore('',nPunteggio, nCellRow, fnMosseFatte);
  oHigh.ShowModal;
  oHigh.Free;
  oHigh := nil;
  if lNew then
  begin
    CleanArray;
    NewNumber;
    NewNumber;
    DrawMap(nCellRow);
    FillRect;
  end;
end;

Function TFMain.SetColor(nNum: Integer): TAlphaColor;
begin
  Result := TAlphaColors.Alpha or TAlphaColor($FF778F)
end;

procedure TFMain.FillRect;
var
  nNumberObj: Integer;
  y: Integer;
  x: Integer;
  oComp: TComponent;
  color: TAlphaColor;
begin
  nNumberObj := 0;
  for y := 1 TO nCellRow do
  begin
    for x := 1 TO nCellRow do
    begin
      oComp := FindComponent('M'+nNumberObj.ToString);
      if assigned(oComp) then
      begin
        (oComp as TPlayTile).Canvas.BeginScene;
        (oComp as TPlayTile).Opacity := 1;
        (oComp as TPlayTile).Tag := aGrid[y][x];
        color := SetColor((oComp as TPlayTile).Tag);
        if (oComp as TPlayTile).Tag = 2 then
          color := TAlphaColors.Crimson
        else if (oComp as TPlayTile).Tag = 4 then
          color := TAlphaColors.Alpha or TAlphaColor($FF778F)//FF778F) //16742287
        else if (oComp as TPlayTile).Tag = 8 then
          color := TAlphaColors.Alpha or TAlphaColor($C955CC) //C955CC) //13194700
        else if (oComp as TPlayTile).Tag = 16 then
          color := TAlphaColors.Alpha or TAlphaColor($E39DF4) //E39DF4) //14917108
        else if (oComp as TPlayTile).Tag = 32 then
          color := TAlphaColors.Alpha or TAlphaColor($8852ED) //8852ED)
        else if (oComp as TPlayTile).Tag = 64 then
          color := TAlphaColors.Alpha or TAlphaColor($6C72B3)
        else if (oComp as TPlayTile).Tag = 128 then
          color := TAlphaColors.Alpha or TAlphaColor($02BF20)
        else if (oComp as TPlayTile).Tag = 256 then
          color := TAlphaColors.Alpha or TAlphaColor($B3B92F)
        else if (oComp as TPlayTile).Tag = 512 then
          color := TAlphaColors.Alpha or TAlphaColor($8AB3E3)
        else if (oComp as TPlayTile).Tag = 1024 then
          color := TAlphaColors.Alpha or TAlphaColor($0AFC80)
        else if (oComp as TPlayTile).Tag = 2048 then
          color := TAlphaColors.Alpha or TAlphaColor($8AB3E3)
        else if (oComp as TPlayTile).Tag = 4096 then
          color := TAlphaColors.Alpha or TAlphaColor($8AB3E3)
        else
        begin
          color := TAlphaColors.Azure;
          (oComp as TPlayTile).Opacity := 0.2;
        end;
       (oComp as TPlayTile).Fill.Color := color;
       (oComp as TPlayTile).Canvas.EndScene;
      end;
      inc(nNumberObj);
    end;
  end;
end;

procedure TFMain.Rectangle1Click(Sender: TObject);
begin
  Close;
end;

procedure TFMain.Rectangle1Paint(Sender: TObject; Canvas: TCanvas;  const [Ref] ARect: TRectF);
var
  color: TAlphaColor;
  sTag: String;
begin
 if (Sender as TPlayTile).Tag > 0 then
  begin
    Canvas.BeginScene;
    Canvas.Font.Family := 'Modern';
    case nCellRow of
      CS_MINROW:  Canvas.Font.Size := 30;
      CS_MEDROW:  Canvas.Font.Size := 20;
      CS_MAXROW:  Canvas.Font.Size := 10;
    end;
    Canvas.Font.Style := [TFontStyle.fsBold, TFontStyle.fsItalic];
    sTag := (Sender as TPlayTile).Tag.ToString;
    Canvas.FillText(ARect, sTag , true,100, [TFillTextFlag.ftRightToLeft], TTextAlign.taCenter, TTextAlign.taCenter);
    Canvas.EndScene;
  end;
end;

procedure TFMain.Rectangle3Click(Sender: TObject);
begin
  HighScore(False);
end;

procedure TFMain.btAltoClick(Sender: TObject);
begin
  MoveSXDX(2, nCellRow, -1, tpAlto);
end;

procedure TFMain.btsinistraClick(Sender: TObject);
begin
  MoveSXDX(2, nCellRow, -1, tpSinistra);
end;

Procedure TFMain.MoveSXDX(nLimite, nFrom, nPasso: Integer; oDirezione: tpDirezione);
var x, y, nValueX, nValueY: Integer;
  lMoved, lFlag: Boolean;
  destX, destY: Integer;
  dx: Integer;
  numt: Integer;
  z: Integer;
  nDest: Integer;
  Function GetDestinazione: Integer;
  begin
    while True do
    begin
      case oDirezione of
        tpDestra, tpSinistra: Result := aGrid[y][dx];
        tpBasso, tpAlto: Result := aGrid[dx][y];
      end;
      IF (Result <> 0) or ( dx = nLimite + nPasso) THEN break;   //  per dx e sx
      dx := dx + nPasso;
      lmoved := True;
    end;
  end;

  Function GetValue: Integer;
  begin
      case oDirezione of
        tpDestra, tpSinistra: Result := aGrid[y][x];
        tpBasso, tpAlto: Result := aGrid[x][y];
      end;
  end;

  Function SetValue(nValore, nPos: Integer): Integer;
  begin
      case oDirezione of
        tpDestra, tpSinistra: begin
                                aGrid[y][x] := 0;
                                aGrid[y][nPos] := nValore;
                              end;
        tpBasso, tpAlto: begin
                           aGrid[x][y] := 0;
                           aGrid[nPos][y] := nValore;
                         end;
      end;
  end;

begin
   for y := 1 TO nCellRow do
   begin
     x := nLimite;
     lFlag := true;
     while lFlag do
     begin
       nValueX := GetValue;
       IF nValueX <> 0 then   // per dx e sx
       begin
         dx := x + nPasso;
         nDest := GetDestinazione;
         if nDest = 0 then
         begin
           SetValue(nValueX, dx);
           lmoved := True;
         end
         else if nDest = nValueX then
         begin
           SetValue(-2 * nValueX, dx);
           nPunteggio := nPunteggio + 2 * nValueX;
           nMosse := nMosse - 1;
           lmoved := True;
         end
         else
         begin
           SetValue(nValueX, dx-npasso);
         end;
       end;
       x := x - nPasso;
       case oDirezione of
         tpDestra, tpBasso:   lFlag := not(x < nFrom);
         tpSinistra, tpAlto: lFlag := not(x > nFrom);
       end;
     end;
     for z:=1 TO nCellRow do
       case oDirezione of
         tpDestra, tpSinistra:   aGrid[y][z] := ABS(aGrid[y][z]);
         tpBasso, tpAlto: aGrid[z][y] := ABS(aGrid[z][y]);
       end;
   end;
   if lmoved then
   begin
     if nMosse <> nCellRow*nCellRow then
       NewNumber;
     FillRect;
     Layout1.Repaint;
   end
   else
   begin
     if (nMosse = nCellRow*nCellRow) and NoMore then
       HighScore(True);
   end;
end;

procedure TFMain.btDestraClick(Sender: TObject);
begin
// destra
   MoveSXDX(nCellRow - 1, 1, 1, tpDestra);
end;


procedure TFMain.btBassoClick(Sender: TObject);
begin
//basso
   MoveSXDX(nCellRow - 1, 1, 1, tpBasso);
end;

procedure TFMain.SetnMosse(const Value: Integer);
begin
  fnMosse := Value;
  lMosse.Text := Value.ToString;
end;

procedure TFMain.SetnMosseFatte(const Value: Integer);
begin
  fnMosseFatte := Value;
  lMosseFatte.Text := Value.ToString;
end;

procedure TFMain.SetnPunteggio(const Value: Integer);
begin
  fnPunteggio := Value;
  lPunteggio.Text := Value.ToString;
end;

end.



 2=FF0000
 4=FF00AA
 8=FFAA00

 16=00FF00
 32=00FFAA
 64=AAFF00

 128=0000FF
 256=00AAFF
 512=AA00FF

 1024=FF5555
 2048=FF55EE
 4096=FFEE55

8192
16384
32768
65536
131072
262144
524288
1048576

