
unit uHighScores;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
  FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Stan.Async,
  FireDAC.Phys, FireDAC.Phys.SQLite, FireDAC.Stan.ExprFuncs, FireDAC.Stan.Param,
  FireDAC.DatS, FireDAC.DApt.Intf, FireDAC.DApt, FireDAC.FMXUI.Wait,
  Data.Bind.EngExt, FMX.Bind.DBEngExt, System.Rtti, System.Bindings.Outputs,
  FMX.Bind.Editors, Data.Bind.Components, Data.Bind.DBScope, FireDAC.Comp.UI,
  FireDAC.Comp.Client, Data.DB, FireDAC.Comp.DataSet, FMX.Layouts, FMX.ListBox,
  FMX.Objects, FMX.ListView.Types, FMX.ListView, FireDAC.Phys.SQLiteDef,
  FMX.Controls.Presentation, FMX.Edit, FMX.ListView.Appearances,
  FMX.ListView.Adapters.Base, FMX.Grid.Style, Fmx.Bind.Grid, Data.Bind.Grid,
  FMX.ScrollBox, FMX.Grid;

type
  TFrameHighScores = class(TForm)
    FireScoresList: TFDConnection;
    FDTableHighScores: TFDTable;
    FDQueryDelete: TFDQuery;
    FDQueryInsert: TFDQuery;
    FDPhysSQLiteDriverLink1: TFDPhysSQLiteDriverLink;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    ContinueBTN: TButton;
    Rectangle1: TRectangle;
    FDTableHighScoresName: TWideMemoField;
    FDTableHighScoresScore: TIntegerField;
    Label1: TLabel;
    InputBGRect: TRectangle;
    InputBoxRect: TRectangle;
    Label2: TLabel;
    InputEdit: TEdit;
    GridPanelLayout1: TGridPanelLayout;
    OkayBTN: TButton;
    CancelBTN: TButton;
    Line1: TLine;
    FDTableHighScoresGrid: TIntegerField;
    Grid1: TGrid;
    BindSourceDB1: TBindSourceDB;
    BindingsList1: TBindingsList;
    LinkGridToDataSourceBindSourceDB1: TLinkGridToDataSource;
    Rectangle3: TRectangle;
    Rectangle2: TRectangle;
    FDTableHighScoresMosse: TIntegerField;
    procedure FireScoresListAfterConnect(Sender: TObject);
    procedure FireScoresListBeforeConnect(Sender: TObject);
    procedure OkayBTNClick(Sender: TObject);
    procedure CancelBTNClick(Sender: TObject);
    procedure ContinueBTNClick(Sender: TObject);
  private
    { Private declarations }
    Loaded: Boolean;
    fnCellRow: integer;
    fnMosse: Integer;
  public
    { Public declarations }
    procedure AddScore(Name: String; Score, Grid, Mosse: Integer);
    procedure SaveScore(Name: String; Score, Grid, Mosse: Integer);
    procedure InitFrame;
    procedure CloseInputBox;
  end;

implementation

uses
  IOUtils;

{$R *.fmx}

procedure TFrameHighScores.CancelBTNClick(Sender: TObject);
begin
  CloseInputBox;
end;

procedure TFrameHighScores.FireScoresListAfterConnect(Sender: TObject);
begin
  FireScoresList.ExecSQL
    ('CREATE TABLE IF NOT EXISTS HighScores (Name TEXT NOT NULL, Score INTEGER, Grid INTEGER, Mosse Integer)');
end;

procedure TFrameHighScores.FireScoresListBeforeConnect(Sender: TObject);
begin
  FireScoresList.Params.Values['Database'] := TPath.GetDocumentsPath + PathDelim
    + 'HighScores.s3db';
end;

procedure TFrameHighScores.AddScore(Name: String; Score, Grid, Mosse: Integer);
begin
  if Name.Trim <> '' then
  begin
    SaveScore(Name, Score, Grid, Mosse);
  end
  else
  begin
    fnCellRow := Grid;
    fnMosse := Mosse;
    InputEdit.Tag := Score;
    InputBGRect.Visible := True;
    InputBGRect.BringToFront;
    InputBoxRect.Visible := True;
    InputBoxRect.BringToFront;
  end;
end;

procedure TFrameHighScores.SaveScore(Name: String; Score, Grid, Mosse: Integer);
begin
  if (Name.Trim <> '') then
  begin
    FDQueryInsert.ParamByName('Name').AsString := Name;
    FDQueryInsert.ParamByName('Score').AsInteger := Score;
    FDQueryInsert.ParamByName('Grid').AsInteger := Grid;
    FDQueryInsert.ParamByName('Mosse').AsInteger := Mosse;
    FDQueryInsert.ExecSQL();
    FDTableHighScores.Refresh;
  end;
end;

procedure TFrameHighScores.InitFrame;
begin
  if Loaded = False then
  begin
    try
      FireScoresList.Connected := True;
      FDTableHighScores.Active := True;
    except
      on e: Exception do
      begin
        ShowMessage(e.Message);
      end;
    end;
    Loaded := True;
  end;
end;

procedure TFrameHighScores.CloseInputBox;
begin
  InputBGRect.Visible := False;
  InputBoxRect.Visible := False;
end;

procedure TFrameHighScores.ContinueBTNClick(Sender: TObject);
begin
  close;
end;

procedure TFrameHighScores.OkayBTNClick(Sender: TObject);
begin
  CloseInputBox;
  SaveScore(InputEdit.Text, InputEdit.Tag, fnCellRow, fnMosse);
end;

end.
